package com.zanxiang.game.module.base.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
 * Bean 工具类
 *
 * @author ruoyi
 */
public class BeanUtils extends org.springframework.beans.BeanUtils {

    /**
     * 对象浅拷贝
     *
     * @param source 源
     * @param target 目标
     * @return {@link T}
     */
    public static <T> T copy(Object source, Class<T> target) {
        if (source == null) {
            return null;
        }
        T obj = instantiateClass(target);
        copyProperties(source, obj);
        return obj;
    }

    /**
     * 对象浅拷贝
     *
     * @param sourceList 源列表
     * @param target     目标
     * @return {@link List}<{@link T}>
     */
    public static <T> List<T> copyList(Collection sourceList, Class<T> target) {
        if (sourceList == null || sourceList.isEmpty()) {
            return Collections.emptyList();
        }
        List<T> objs = new ArrayList<>(sourceList.size());
        for (Object source : sourceList) {
            objs.add(copy(source, target));
        }
        return objs;
    }
}
